package com.nisum.ascend.inventory.service;
import com.nisum.ascend.inventory.dto.ProductDto;
import com.nisum.ascend.inventory.dto.PromotionDto;
import com.nisum.ascend.inventory.exception.ResourceNotFoundException;
import com.nisum.ascend.inventory.model.Product;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import com.nisum.ascend.inventory.repository.ProductRepository;
import reactor.core.publisher.Mono;

import java.util.ArrayList;
import java.util.List;


@Service
public class ProductService {
    @Autowired
    ProductRepository productRepository;

    public ProductDto generateDtoFromProduct(Product product) {
        // TODO: Fetch the product's promotion from the Promotions com.nisum.ascend.inventory.service
        List<PromotionDto> promotions = new ArrayList<>();

        return new ProductDto(product.getSku(), product.getUpc(), product.getProductName(), product.getProductDescription(), product.getPrice(), product.getProductImageUrl(), product.getBrand(), product.getCategory(), product.getAvailableStock(), promotions);

    }

    public Mono<ProductDto> removeProductBySku(String sku) {
        return productRepository
                .findBySku(sku)
                .flatMap(existingProduct -> productRepository.delete(existingProduct)
                        .then(Mono.just(existingProduct)))
                        .map(this::generateDtoFromProduct)
                .switchIfEmpty(Mono.error(new ResourceNotFoundException(HttpStatus.NOT_FOUND, "Product Not Found")));
    }

    public Mono<ProductDto> getProductBySku(String sku){
        return productRepository.findBySku(sku)
                .map(this::generateDtoFromProduct)
                .switchIfEmpty(Mono.error(new ResourceNotFoundException(HttpStatus.NOT_FOUND, "product not found")));
    }

    public Flux<ProductDto> findAllProducts() {
        return productRepository.findAll()
                .map(this::generateDtoFromProduct);
    }

    public Mono<ProductDto> updateProductBySku(String sku,  Product product){
        return productRepository.findBySku(sku)
                .flatMap(dbProduct -> {
                    dbProduct.setUpc(product.getUpc());
                    dbProduct.setProductName(product.getProductName());
                    dbProduct.setProductDescription(product.getProductDescription());
                    dbProduct.setBrand(product.getBrand());
                    dbProduct.setCategory(product.getCategory());
                    dbProduct.setPrice(product.getPrice());
                    dbProduct.setAvailableStock(product.getAvailableStock());
                    dbProduct.setProductImageUrl(product.getProductImageUrl());
                    return productRepository.save(dbProduct);
                }).map(this::generateDtoFromProduct);
    }

    public Mono<ProductDto> createProduct(Product product) {
        return productRepository.save(product)
                .map(this::generateDtoFromProduct);
    }

}
